package de.gematik.ti.vauchannel.protocol;

import com.google.common.base.Charsets;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.nio.ByteBuffer;

import static de.gematik.ti.vauchannel.protocol.VAUProtocolException.SYNTAX_ERROR;
import static de.gematik.ti.vauchannel.protocol.helpers.VAUProtocolHelpers.concat;

public class TransportedData {


    private final Logger log = LoggerFactory.getLogger(this.getClass());

    public byte[] body;
    public String contentType;

    public TransportedData() {

    }

    public TransportedData(byte[] body, String contentType) {
        this.body = body;
        this.contentType = contentType;
        log();
    }

    public static TransportedData fromRaw(byte[] raw) {
        byte[] numberOfBytes_inBytes = new byte[4];
        System.arraycopy(raw, 0, numberOfBytes_inBytes, 0, 4);
        int numberOfBytes = java.nio.ByteBuffer.wrap(numberOfBytes_inBytes).getInt();
        byte[] headerField_inBytes = new byte[numberOfBytes];
        System.arraycopy(raw, 4, headerField_inBytes, 0, numberOfBytes);
        String headerField = new String(headerField_inBytes, Charsets.US_ASCII);
        int i = headerField.indexOf(":");

        String value = headerField.substring(i + 1, headerField.length()).trim();
        String key = headerField.substring(0, i);

        if (!key.equalsIgnoreCase("Content-Type")) {
            throw new VAUProtocolException(SYNTAX_ERROR);
        }
        TransportedData td = new TransportedData();
        td.contentType = value;
        td.body = new byte[raw.length - 4 - numberOfBytes];
        System.arraycopy(raw, 4 + numberOfBytes, td.body, 0, td.body.length);
        td.log();
        return td;
    }

    public byte[] getRaw() {

        String headerField = "Content-Type: " + contentType;
        byte[] headerField_inBytes = headerField.getBytes(Charsets.US_ASCII);
        byte[] numberOfBytes_inBytes = ByteBuffer.allocate(4).putInt(headerField_inBytes.length).array();
        return concat(concat(numberOfBytes_inBytes, headerField_inBytes), this.body);
    }


    public void log() {
        log.info("contentType: " + contentType + "; body length: " + this.body.length);
        log.debug(new String(this.body, Charsets.UTF_8));
    }
}
